@@ -177,7 +177,6 @@ class Workflow(PegaWorkflow):
     # --- Write files in directory -------------------------------------------------
     def write(self, output_path=None, **keywords):
         """
-        TODO: write properties, site_catalog, replica_catalog to files in the workflow folder.
         """
         ## Pegasus properties will be in the static file.
         #self.pegasus_props.write()
@@ -770,13 +769,16 @@ class Workflow(PegaWorkflow):
         transfer is equivalent to stage_out, only for output file.
         """
         if is_input:
-            if hasattr(job, "inputLs"):
-                job.inputLs.append(file)
-            job.add_inputs(file)
+            if not job.has_input_file(file):
+                if hasattr(job, "inputLs"):
+                    job.inputLs.append(file)
+                job.add_inputs(file)
         else:
-            if hasattr(job, "outputLs"):
-                job.outputLs.append(file)
-            job.add_outputs(file, stage_out=transfer, register_replica=register_replica)
+            if not job.has_output_file(file):
+                if hasattr(job, "outputLs"):
+                    job.outputLs.append(file)
+                job.add_outputs(file, stage_out=transfer,
+                    register_replica=register_replica)
         return True
 
     def addJobDependency(self, parentJob=None, childJob=None):
@@ -827,7 +829,8 @@ class Workflow(PegaWorkflow):
         key2ObjectForJob=None, objectWithDBArguments=None,
         objectWithDBGenomeArguments=None,\
         no_of_cpus=None, job_max_memory=200, walltime=180,
-        max_walltime=None, **keywords):
+        max_walltime=None, 
+        gpu=None, **keywords):
         """
         A generic job adding function for other functions to use.
         The commandline order:
@@ -910,7 +913,7 @@ class Workflow(PegaWorkflow):
 
         setJobResourceRequirement(job, job_max_memory=job_max_memory,
             sshDBTunnel=sshDBTunnel,\
-            no_of_cpus=no_of_cpus, walltime=walltime)
+            no_of_cpus=no_of_cpus, walltime=walltime, gpu=gpu)
         self.add_jobs(job)
         job.parentJobLs = []
         if parentJob:
@@ -1112,8 +1115,8 @@ fastaDictJob = self.addGenericJavaJob(executable=CreateSequenceDictionaryJava,
             flush=True)
         return
 
-    def getJVMMemRequirment(self, job_max_memory=5000, minMemory=500,
-        permSizeFraction=0, MaxPermSizeUpperBound=35000):
+    def getJVMMemRequirment(self, job_max_memory:int=5000, minMemory:int=500,
+        permSizeFraction:int=0, MaxPermSizeUpperBound=35000):
         """
         Java 8 does not support PermSize anymore. set permSizeFraction to 0.
         handle when job_max_memory is None and minMemory is None.
@@ -1154,7 +1157,7 @@ fastaDictJob = self.addGenericJavaJob(executable=CreateSequenceDictionaryJava,
         return PassingData(memRequirementInStr=memRequirementInStr,
             memRequirement=memRequirement)
 
-    def scaleJobWalltimeOrMemoryBasedOnInput(self, realInputVolume=10, \
+    def scaleJobWalltimeOrMemoryBasedOnInput(self, realInputVolume:int=10, \
         baseInputVolume=4, baseJobPropertyValue=120, \
         minJobPropertyValue=120, maxJobPropertyValue=1440):
         """
@@ -1163,7 +1166,7 @@ fastaDictJob = self.addGenericJavaJob(executable=CreateSequenceDictionaryJava,
         """
         walltime = min(max(minJobPropertyValue, 
             float(realInputVolume)/float(baseInputVolume)*baseJobPropertyValue),
-            maxJobPropertyValue)	#in minutes
+            maxJobPropertyValue)	#in minutes or MB
         return PassingData(value=int(walltime))
 
     def addMkDirJob(self, outputDir=None, executable=None, 
